package com.dxhy.customs.controller;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dxhy.common.constant.CommonConstants;
import com.dxhy.common.controller.AbstractController;
import com.dxhy.common.util.UserInfoUtil;
import com.dxhy.common.utils.R;
import com.dxhy.customs.model.TdxCustomsRecordModelDTO;
import com.dxhy.customs.service.CustomsRecordDkrzService;

import lombok.extern.slf4j.Slf4j;

/**
 * 缴款书认证
 *
 * @author 陈刚扬
 * @date 2020/8/10
 */
@RestController
@RequestMapping("/customsRecordRzdk")
@Slf4j
public class CustomsRecordRzdkController extends AbstractController {
    @Resource
    private CustomsRecordDkrzService tDxCustomsRecordService;

    /**
     * 抵扣认证 - 缴款书手工认证-列表查询
     *
     * @author 陈刚扬
     * @date 2020/8/10
     * @param page
     *            入参
     * @param dto
     *            入参
     * @return org.springframework.http.ResponseEntity<?>
     */
    @PostMapping("/querySgByPage")
    public ResponseEntity<?> querySgByPage(Page<?> page, @RequestBody TdxCustomsRecordModelDTO dto) {
        try {
            return ResponseEntity
                .ok(R.ok().put("data", tDxCustomsRecordService.querySgByPage(page, dto, this.getUserInfo())));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

    /**
     * 缴款书调教认证抵扣、不抵扣
     *
     * @author 陈刚扬
     * @date 2020/8/10
     * @param dto
     *            入参
     * @return org.springframework.http.ResponseEntity<?>
     */
    @PostMapping("/approveSg")
    public ResponseEntity<?> approveSg(@RequestBody TdxCustomsRecordModelDTO dto) {
        try {
            String result = tDxCustomsRecordService.approveSg(dto, this.getUserInfo());
            return ResponseEntity.ok(R.ok().put("data", result));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(e.getMessage()));
        }
    }

    /**
     * 缴款书认证抵扣excel导入校验
     *
     * @author 陈刚扬
     * @date 2020/8/10
     * @param pramsMap
     *            入参
     * @return org.springframework.http.ResponseEntity<?>
     */
    @PostMapping("/importCert")
    public ResponseEntity<?> importCert(@RequestBody Map<String, Object> pramsMap) {
        try {

            List<String> gfshList = UserInfoUtil.getGfshAll(getUserInfo().getOrg());
            pramsMap.put("dbName", getUserInfo().getDbName());

            pramsMap.put("gfsh", gfshList);
            String datalist = (String)pramsMap.get("datalist");
            if (StringUtils.isNotBlank(datalist)) {
                pramsMap.put("datalist", datalist);
            } else {
                return ResponseEntity.ok(R.error("请上传excel文件!"));
            }

            return ResponseEntity.ok(Objects
                .requireNonNull(R.ok().put("data", tDxCustomsRecordService.importCert(pramsMap, getUserInfo()))));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

    /**
     * 文件下载
     *
     */
    @RequestMapping("/downLoad")
    @ResponseBody
    public ResponseEntity<?> downLoadFtpFileMould(String fileName, HttpServletResponse response) {
        try {
            File downLoadFile = new File(this.getClass().getResource("/").getPath() + "template/" + fileName);
            // 获取文件的长度
            long fileLength = downLoadFile.length();
            BufferedInputStream bis;
            BufferedOutputStream bos;

            // 设置文件输出类型
            response.setContentType("application/octet-stream");
            response.setHeader("Content-disposition",
                "attachment; filename=" + new String(fileName.getBytes(StandardCharsets.UTF_8), "ISO8859-1"));
            // 设置输出长度
            response.setHeader("Content-Length", String.valueOf(fileLength));
            // 获取输入流
            bis = new BufferedInputStream(new FileInputStream(downLoadFile));
            // 输出流
            bos = new BufferedOutputStream(response.getOutputStream());
            byte[] buff = new byte[2048];
            int bytesRead;
            while (-1 != (bytesRead = bis.read(buff, 0, buff.length))) {
                bos.write(buff, 0, bytesRead);
            }
            bos.flush();
            // 关闭流
            bis.close();
            bos.close();
            return null;
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

    /**
     * 抵扣认证 - 缴款书设置不抵扣-列表查询
     */
    @PostMapping("/queryBdkByPage")
    public ResponseEntity<?> queryBdkByPage(Page<?> page, @RequestBody TdxCustomsRecordModelDTO dto) {
        try {
            return ResponseEntity
                .ok(R.ok().put("data", tDxCustomsRecordService.queryBdkByPage(page, dto, this.getUserInfo())));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

    /**
     * 抵扣认证 - 缴款书认证查询-列表查询
     */
    @PostMapping("/queryRzcxByPage")
    public ResponseEntity<?> queryRzcxByPage(Page<?> page, @RequestBody TdxCustomsRecordModelDTO dto) {
        try {
            return ResponseEntity
                .ok(R.ok().put("data", tDxCustomsRecordService.queryRzcxByPage(page, dto, this.getUserInfo())));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

    /**
     * 抵扣认证 - 缴款书认证查询-撤销抵扣或撤销不抵扣
     */
    @PostMapping("/approveCxrz")
    public ResponseEntity<?> approveCxrz(@RequestBody TdxCustomsRecordModelDTO dto) {
        try {
            String result = tDxCustomsRecordService.approveCxrz(dto, this.getUserInfo());
            return ResponseEntity.ok(R.ok().put("data", result));
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(R.error(e.getMessage()));
        }
    }
}
